/******************************************************************************
   FILE:
      skiplist.h

   DESCRIPTION:
      Header file for skip list implementation.  The skiplist is an ADT
      that tests equally as well as a binary tree, though easier to
      implement and understand.

   NOTES:
      The SkipList ADT and implementation created by William Pugh 1987.
      Code was examined via ftp from minsy.umd.edu.  This code differs in 
      that instead of a local update vector, it utilizes an update vector 
      in the header, which is used to aid the insert and delete functions. 
      Unlike Pugh's code, this code does not assume valid data and correct
      implementation by the user.

   DEVELOPED BY:                                 DATE:
      James E. Pitkow                               July 30, 1993
      Undergraduate Research Assistant
      Laboratory for Atmospheric and Space Physics
      Campus Box 590
      Boulder, CO 80309 USA
      pitkow@aries.colorado.edu

   COPYRIGHT:
      The Regents of the University of Colorado, 1993

******************************************************************************/

#include <stdio.h>
#include "defs.h"

#ifndef SKIPLIST_H
#define SKIPLIST_H

/* declare constants through out implementation */
#define MAX_HEIGHT  17		 /* height of skiplist starting from one */
#define MAX_LEVEL   MAX_HEIGHT-1 /* this will create levels 0 to MAX_HEIGHT -1*/

typedef struct skip_node
{
    unsigned short    height;    /* height of node                          */
    Datum            *data;      /* pointer to user defined data            */
    struct skip_node *forward[1];/* dynamic array pointing to next nodes    */
} Skip_Node;

typedef struct skip_list
{
   int        level;             /* maximum height of the list               */
   int        length;            /* up-to-date length of list                */
   int        num_search;        /* tracks number of nodes searched in find  */
   int       (*compare)();       /* used defined compare function for data   */
   float      prob;              /* probability of using new level           */
   Skip_Node *update[MAX_LEVEL]; /* update vector used to enhance code       */
   Skip_Node *first;             /* pointer to first node                    */
} Skip_List;

/* define macros */
#define skiplist_length(sk)     (!(sk)->first?-1:(sk)->length )
#define skiplist_getdata(node)  (!(node)?(Datum*)NULL:(Datum*)(node)->data )
#define skiplist_first(sk)      (!(sk)->first->forward[0]? (Skip_Node*)NULL:\
                                 (sk)->first->forward[0] )
#define skiplist_next(node)     (!(node)?(Skip_Node*)NULL:(node)->forward[0] )
#define skiplist_getlevel(node) (!(node)?-1:((node)->height + 1) )

/* define implemented functions */           
Skip_List     *skiplist_create  ( );
Skip_Node     *skiplist_find    ( );
Skip_Node     *skiplist_insert  ( );
Datum         *skiplist_delete  ( );
void           skiplist_destroy ( );
MY_BOOLEAN     skiplist_apply   ( );
unsigned short skiplist_genlevel( );

#endif
