/*		FILE WRITER				HTFWrite.h
**		===========
**
**	This version of the stream object just writes to a C file.
**	The file is assumed open and left open.
**
**	Bugs:
**		strings written must be less than buffer size.
*/

#include "HTFWriter.h"

#include "HTFormat.h"
#include "HTAlert.h"
#include "HTFile.h"
#include "HText.h"

/*		Stream Object
**		------------
*/

struct _HTStream {
	CONST HTStreamClass *	isa;
	
	FILE *			fp;
	char * 			end_command;
};

/* #define TRACE 1 */

/* MOSAIC: We now pick up some external variables, handled
   in src/mo-www.c: */
extern int force_dump_to_file;
extern char *force_dump_filename;
/* If force_dump_to_file is high, we know we want to dump the
   data into a file already named by force_dump_filename and not
   do anything else. */

/*_________________________________________________________________________
**
**			A C T I O N 	R O U T I N E S
*/

/*	Character handling
**	------------------
*/

PRIVATE void HTFWriter_put_character ARGS2(HTStream *, me, char, c)
{
    putc(c, me->fp);
}



/*	String handling
**	---------------
**
**	Strings must be smaller than this buffer size.
*/
PRIVATE void HTFWriter_put_string ARGS2(HTStream *, me, CONST char*, s)
{
    fputs(s, me->fp);
}


/*	Buffer write.  Buffers can (and should!) be big.
**	------------
*/
PRIVATE void HTFWriter_write ARGS3(HTStream *, me, CONST char*, s, int, l)
{
#if 0
  if (TRACE)
    fprintf (stderr, "[HTFWriter_write] Writing %d bytes\n", l);
#endif
  fwrite(s, 1, l, me->fp); 
}




/*	Free an HTML object
**	-------------------
**
**	Note that the SGML parsing context is freed, but the created
**	object is not,
**	as it takes on an existence of its own unless explicitly freed.
*/
PRIVATE void HTFWriter_free ARGS1(HTStream *, me)
{
  HText *text;

  fflush(me->fp);

  if (force_dump_to_file)
    {
      if (TRACE)
        fprintf (stderr, "[HTFWriter_free] Closing force_dump_to_file's file and DONE.\n");
      fclose (me->fp);
      goto done;
    }

  /* Now, me->end_command can either be something starting with
     "<mosaic-internal-reference" or it can be a real command.
     Deal with appropriately. */
  if (me->end_command)
    {
      if (strncmp (me->end_command, "<mosaic-internal-reference", 26))
        {
          if (TRACE)
            fprintf (stderr, "[HTFWriter_free] Doing command '%s'\n", me->end_command);
          fclose(me->fp);
          HTProgress("Spawning external viewer.");	/* Tell user what's happening */
          system(me->end_command);
          free (me->end_command);
        }
      else
        {
          /* Internal reference, aka HDF file.  Just close output file. */
          fclose (me->fp);
        }
    }
  else
    {
      /* No me->end_command; just close the file. */
      fclose (me->fp);
    }
  
  /* Construct dummy HText thingie so Mosaic knows
     not to try to access this "document". */
  text = HText_new (NULL);
  HText_beginAppend (text);
  /* If it's a real internal reference, tell Mosaic. */
  if (me->end_command)
    {
      if (!strncmp (me->end_command, "<mosaic-internal-reference", 26))
        {
          if (TRACE)
            fprintf (stderr, "YEAH BABY YEAH! '%s'\n", me->end_command);
          HText_appendText (text, me->end_command);
        }
      else
        {
          HText_appendText (text, "<mosaic-access-override>\n");
        }
      free (me->end_command);
    }
  else
    {
      /* No me->end_command; just override the access. */
      HText_appendText (text, "<mosaic-access-override>\n");
    }
  HText_endAppend (text);

 done:
  free(me);

  return;
}

/*	End writing
*/

PRIVATE void HTFWriter_end_document ARGS1(HTStream *, me)
{
    fflush(me->fp);
}



/*	Structured Object Class
**	-----------------------
*/
PRIVATE CONST HTStreamClass HTFWriter = /* As opposed to print etc */
{		
	"FileWriter",
	HTFWriter_free,
	HTFWriter_end_document,
	HTFWriter_put_character, 	HTFWriter_put_string,
	HTFWriter_write
}; 


/*	Subclass-specific Methods
**	-------------------------
*/

PUBLIC HTStream* HTFWriter_new ARGS1(FILE *, fp)
{
    HTStream* me;
    
    if (!fp) return NULL;

    me = (HTStream*)malloc(sizeof(*me));
    if (me == NULL) outofmem(__FILE__, "HTFWriter_new");
    me->isa = &HTFWriter;       

    me->fp = fp;
    me->end_command = NULL;

    return me;
}



/*	Take action using a system command
**	----------------------------------
**
**	originally from Ghostview handling by Marc Andreessen.
**	Creates temporary file, writes to it, executes system command
**	on end-document.  The suffix of the temp file can be given
**	in case the application is fussy, or so that a generic opener can
**	be used.
**
**      WARNING: If force_dump_to_file is high, pres may be NULL
**      (as we may get called directly from HTStreamStack).
*/
PUBLIC HTStream* HTSaveAndExecute ARGS3(
	HTPresentation *,	pres,
	HTParentAnchor *,	anchor,	/* Not used */
	HTStream *,		sink)	/* Not used */
{
  char *fnam;
  char *command;
  CONST char * suffix;
  
  HTStream* me;
  
  me = (HTStream*)malloc(sizeof(*me));
  if (me == NULL) outofmem(__FILE__, "Save and execute");
  me->isa = &HTFWriter;  
  
  /* Save the file under a suitably suffixed name */
  
  if (!force_dump_to_file)
    {
      suffix = HTFileSuffix(pres->rep);
      
      fnam = (char *)malloc (L_tmpnam + 16 + strlen(suffix));
      tmpnam (fnam);
      if (suffix) strcat(fnam, suffix);
    }
  else
    {
      if (TRACE)
        fprintf (stderr, "[HTSaveAndExecute] Duping force_dump_filename '%s'\n",
                 force_dump_filename);
      fnam = strdup (force_dump_filename);
    }

  me->fp = fopen (fnam, "w");
  if (!me->fp) 
    {
      HTAlert("Can't open temporary file!");
      free(fnam);
      free(me);
      return NULL;
    }
  if (TRACE)
    fprintf (stderr, "[HTSaveAndExecute] Opened tmp file '%s'\n", fnam);

  /* If force_dump_to_file is high, we're done here. */
  if (!force_dump_to_file)
    {
      if (strcmp (pres->command, "mosaic-internal-reference"))
        {
          /* Make command to process file */
          command = (char *)malloc ((strlen (pres->command) + 10 + 3*strlen(fnam))
                                    * sizeof (char));
          if (command == NULL) outofmem(__FILE__, "SaveAndExecute");
          /* Cute.  pres->command will be something like "xv %s"; fnam
             gets filled in as many times as appropriate.  */
          sprintf (command, pres->command, fnam, fnam, fnam);
          
          me->end_command = (char *)malloc (
                                            (strlen (command) + 32 + strlen(fnam))
                                            * sizeof (char));
          sprintf (me->end_command, "(%s ; /bin/rm -f %s) &",
                   command, fnam);
          if (me->end_command == NULL) outofmem(__FILE__, "SaveAndExecute");
          
          free (command);
        }
      else
        {
          /* Overload me->end_command to be what we should write out as text
             to communicate back to client code. */
          if (TRACE)
            fprintf (stderr, "*** Picked up that we're dealing with an internal reference.\n");
          me->end_command = (char *)malloc
            (strlen (pres->command) + strlen (fnam) + 32);
          if (me->end_command == NULL) outofmem(__FILE__, "SaveAndExecute");
          sprintf (me->end_command, "<%s \"%s\">\n", pres->command, fnam);
          if (TRACE)
            fprintf (stderr, "*** me->end_command '%s'\n", me->end_command);
        }
    }
  
  free (fnam);
  
  return me;
}
