/******************************************************************************
   FILE:
      analyzer_tools.c

   DESCRIPTION:
      This file contains the utility functions for html_analyzer.c  
      Functions are:

	check_files() -> returns status of http or relative addressed links
	validate() -> calls check_files(); prints results
	completeness() -> points out contents that are not associated to a link
	consistent_link() -> notify if multiple contents point to the same link
        cmp_data_link()-> compare based on link than contents values
	consistent_content() -> notify if multiple links are pointed to by a content 
        cmp_data_content()-> compare based on content than link values
        init_data() -> innitializes fields of data structs
        copy_data() -> copies the fields of a data struct into another one
        
      Global varaibles used:
	char *respostiory;   

   DEVELOPED BY:                                 DATE:
      James E. Pitkow                               October 7, 1993
      Graphics, Visualization and Usability Laboratory
      Georgia Institute of Technology
      pitkow@cc.gatech.edu

   COPYRIGHT:
      The Regents of the University of Colorado, 1993
                
******************************************************************************/
#include "html_analyzer.h"

extern char	*repository;

Skip_Data * init_data( )
{
   Skip_Data *data;
   if( !( data = ( Skip_Data *) malloc( ( unsigned) sizeof( Skip_Data)))) {
      fprintf( stderr, "\ninit_data: malloc error");
      safe_bail_out( -1);
   }
   data->href[0] = data->content[0] = data->file_name[0] = '\0';
   data->occurences = 0;
   return( data);
}


void copy_data( data1, data2)
Skip_Data *data1;
Skip_Data *data2;
{
   strcpy( data2->content, data1->content);
   strcpy( data2->href, data1->href);
   strcpy( data2->file_name, data1->file_name);
}

int check_file( data)
Skip_Data *data;
{
   struct  stat	 stat_buffer;
   REG char 	*temp_href;
   REG int	 count;

   /*
    * construct the full address of the link, if necessary
    */
   if ( data->href[0] == '.' ) {   
      for ( count=strlen( data->file_name); 
          ( data->file_name[--count] != '/'););

      if ( !(temp_href=malloc( sizeof(char)*(++count+strlen(data->href)+1)))) {
         fprintf( stderr, "\ncheck_file: can't malloc");
         safe_bail_out( -1);
      }

      bzero( temp_href, (count+strlen(data->href)+1));
      strncpy( temp_href, data->file_name, count);
      strcat( temp_href, data->href);
   }
   else
      temp_href = data->href;

   /*
    * if the link is not locally relative, use the modified http functions 
    * to check the link
    */
   if ( data->href[0] != '.' )
      return HTLoadAbsolute( temp_href);

   /*
    * else, use stat() to verify existence and return the result; 
    * assumes healthy file_name 
    */
   count = stat( temp_href, &stat_buffer);
   free( temp_href);
   if (count == -1)
      return 0;
   return 1;
} 

MY_BOOLEAN validate( data)
Skip_Data *data;
{
   REG int status = check_file( data);

   /* 
    * check_file will perform either a http look-up or verfiry the
    * relative address of the href; this handles the different results
    * of that check; status has the following significance:
    * status ==  1	file exists and is retreivable
    * status ==  0 	server trouble or file does not exist as indicated	
    */
   if ( !status) {
      printf( "\nWARNING:  Failed in checking:\n %s\n", data->href);
      printf( "   With content of: %s\n", data->content);
      printf( "   In local file: %s\n\n", data->file_name);
   }
   else {
      printf( "\nSUCCESS:   File located at:\n %s\n", data->href);
      printf( "   With content of: %s\n", data->content);
      printf( "   In local file: %s\n\n", data->file_name);
   }
   return TRUE;
}

MY_BOOLEAN completeness( data)
Skip_Data *data;
{
   if ( data->content) {
      struct	stat	stat_buffer;
      char	command[BUFSIZ];

      /*
       * perform a caseless grep of the non-html files in 
       * repository/html_analyzer; if the content occurs in a file, it is
       * not a part of an html anchor, but needs to be
       */
      command[0] = '\0';
      sprintf( command, "grep -i -n -l '%s' %s/*.html > temp_file",
                            data->content, repository);
      system( command); 

      /* 
       * check the size of temp_file and show user where unlinked contents are;
       * this does not give the full path names of the files that contain
       * that unlinked contents :-< 
       */
      stat( "temp_file", &stat_buffer);
      if ( stat_buffer.st_size > 1) {
         FILE *temp_fp;
         char  word[BUFSIZ];

         printf( "WARNING: These filenames contain the content:\n");
         printf( "  %s \n", data->content);
         printf( " Without a link to: \n");
         printf( "  %s\n\n", data->href);
         if ( !(temp_fp = fopen( "temp_file", "r"))) {
            fprintf( stderr, "\ncompleteness: unable to open file");
            safe_bail_out( -2);
         }

         /*
          * print out the names of the files that contained the content 
          */
         while ( fscanf( temp_fp, "%s", word) != EOF) {
            char *temp_word = (char *)strrchr( word, '/');
            printf( "%s\n", ++temp_word);
         }
         fclose( temp_fp);
         printf("\n");
      }
      unlink( "temp_file");
   }
   return TRUE;
}
      
int cmp_data_content( d1, d2)
Skip_Data *d1;
Skip_Data *d2;
{
   REG int result = strcmp( d1->content, d2->content); 

   /* 
    * if the contents are the same for both structs, compare the links
    */
   return result ? result : strcmp( d1->href, d2->href);
}
    
int cmp_data_link( d1, d2)
Skip_Data *d1;
Skip_Data *d2;
{
   REG int result = strcmp( d1->href, d2->href);

   /*
    * if the hrefs are the same for both structs, compare the contents
    */
   return result ? result : strcmp( d1->content, d2->content);
}

void consistent_content( sk)
Skip_List *sk;                           
{
    Skip_Node *current;                 
    Skip_Data *data;		

    /* 
     * handle erronous conditions 
     */
    if ( !sk || !sk->first)
        return;

    /* 
     * move through list and check for a one-to-one correspondence between
     * contents used to identify the hrefs and the hrefs 
     */
    current = sk->first->forward[0];
    while ( current) {
        data = (Skip_Data *)current->data;
        current = current->forward[0];
        if ( !current)
           break;
        if ( !strcmp( ( ( Skip_Data *)current->data)->content, data->content)) {

           /* 
            * this means that we have inconsistent usage of the content; e.g. 
            * there are at least two nodes in the skiplist with the same
            * content and different hrefs
            */
           printf("WARNING: Content used inconsistently.\n");
           printf("  CONTENT: %s\n", data->content);
           printf("  occurs %d time%s with href: %s\n", data->occurences,
                       data->occurences > 1 ? "s" : "", data->href);
           printf("  as in file: %s, but also \n", data->file_name);
           printf("  occurs %d time%s with href: %s\n",
                     ( ( Skip_Data *)current->data)->occurences,
                     ( ( Skip_Data *)current->data)->occurences  > 1 ? "s" : "",
                     ( ( Skip_Data *)current->data)->href);
           printf("  as in file: %s\n\n",
                     ( ( Skip_Data *)current->data)->file_name);
        }
    }
}

void consistent_link( sk)
Skip_List *sk;
{
    Skip_Node *current;
    Skip_Data *data;

    /*
     * handle erronous conditions
     */
    if ( !sk || !sk->first)
        return;

    /*
     * move through list and check for a one-to-one correspondence between
     * hrefs and the contents used to identify the hrefs
     */
    current = sk->first->forward[0];
    while ( current) {
        data = (Skip_Data *)current->data;
        current = current->forward[0];
        if ( !current)
           break;
        if ( !strcmp( ( ( Skip_Data *)current->data)->href, data->href)) {

           /*
            * this means that we have inconsistent usage of the href; e.g.
            * there are at least two nodes in the skiplist with the same
            * href and different contents
            */
           printf("WARNING: Link used inconsistently.\n");
           printf("  HREF: %s\n", data->href);
           printf("  occurs %d time%s with content: %s\n", data->occurences,
                       data->occurences > 1 ? "s" : "", data->content);
           printf("  as in file: %s, but also \n", data->file_name);
           printf("  occurs %d time%s with content: %s\n",
                     ( ( Skip_Data *)current->data)->occurences,
                     ( ( Skip_Data *)current->data)->occurences  > 1 ? "s" : "",
                     ( ( Skip_Data *)current->data)->content);
           printf("  as in file: %s\n\n",
                     ( ( Skip_Data *)current->data)->file_name);
        }
    }
}

