/******************************************************************************
   FILE:
      main.c

   DESCRIPTION:
      This file contians the wrapper for html_analyzer().  Simply, it parses
      arguments, creates the temporary repository, calls html_analyzer()
      and then exits.  The functions safe_bail_out and clean_dir were 
      written to enable graceful termination.

      Global variables set:  
         char    *repository;

      Exit status:
         status =  0	normal termination
         status = -1	abnormal termination: malloc error
         status = -2	abnormal termination: fopen error
         status = -3	abnormal termination: misc error

   DEVELOPED BY:                                 DATE:
      James E. Pitkow                               July 30, 1993
      Undergraduate Research Assistant
      Laboratory for Atmospheric and Space Physics
      Campus Box 590
      Boulder, CO 80309 USA
      pitkow@aries.colorado.edu

   COPYRIGHT:
      The Regents of the University of Colorado, 1993

******************************************************************************/
#include "html_analyzer.h"

char 	*repository;


void clean_dir(dir)
char * dir;
{
   DIR           *dir_file_desc;
   struct dirent *dir_pointer;

   /*
    * only process real directories
    */
   if ((dir_file_desc = opendir( dir)) == NULL) 
      return; 

   /*
    * before rmdir(), unlink all files in the directory
    */
   while (dir_pointer = readdir(dir_file_desc))

      /*
       * but do not unlink these two
       */
      if ( !strcmp( dir_pointer->d_name, ".") ||
           !strcmp( dir_pointer->d_name, ".."))
         continue;
      else {
         char *path;
         int   length = strlen( repository) + strlen(dir_pointer->d_name) + 1;

         if ( !(path = malloc( sizeof( char) *length))) {
            fprintf( stderr, "\nclean_fir.c: malloc error");
   /* changed to _exit() Dave Brown 16/3/95; D.M.Brown@csc.liv.ac.uk */
            _exit(-3);
         }

         /*
          * make sure memory is zeroed before calling unlink on contents
          */
         bzero(path, length);
         sprintf(path, "%s/%s", repository, dir_pointer->d_name);
         unlink(path);
      }
   rmdir(repository);
}

 
void safe_bail_out(status)
int status;
{
   /*
    * clean up temporary directory and free used memory
    */
   printf( "\n\nDone Processing\n\n");
   if ( repository != NULL)
      clean_dir( repository);
   free(repository);

   /* changed to _exit() Dave Brown 16/3/95; D.M.Brown@csc.liv.ac.uk */
  _exit(status);
}


main(argc,argv)
int argc;
char **argv;
{
   struct stat	statbuffer;
   short	supress = 0;
   char	*directory,
   	*usage =
     "\nUSAGE: html_analyzer [-val] [-com] [-con] directory [path of repository]\n";

   if ( argc == 1 ) {
      fprintf( stderr, "\nNo directory specified\n%s", usage);
      safe_bail_out( -3);
   }

   /* 
    * parse input, set up flag for testing
    */
   while ( argc > 1 && **(++argv) == '-') {
      switch ( (argv[0][3])++) {
         case 'l': supress |= VALIDATE;
                   break;
         case 'm': supress |= COMPLETE;
                   break;
         case 'n': supress |= CONSISTENT;
                   break;
         default : fprintf( stderr, "\nError specifying supress option\n\n%s",
                            usage);
                   safe_bail_out( -3);
      }
   }

   /* 
    * abort nonsense execution
    */
   if ( supress  == ( VALIDATE | COMPLETE | CONSISTENT)) {
      printf( "\nNo tests requested");
      safe_bail_out( 0);
   }

   /*
    * set the locations of the directory to be processed and the  
    * repository (default: /var/tmp) to create the temporary directory 
    * /html_analyzer from
    */
   if ( !*argv) {
      fprintf( stderr,"\nNo directory specified\n%s", usage);
      safe_bail_out( -3);
   }
   else {
      if ( !( directory = malloc( sizeof( char) * ( strlen( *argv) + 1)))) {
         fprintf( stderr, "\nmain.c: malloc error");
         safe_bail_out( -3);
      }
      strcpy( directory, *argv);
   }

   if ( !*++argv) { 
      if ( !( repository = malloc( sizeof( char) * 23))) {
         fprintf( stderr, "\nmain.c: malloc error");
         safe_bail_out( -3);
      }
      strcpy( repository, "/opt/tmp/html_analyzer");
      fprintf( stderr,"\nUsing default repostitory: %s", repository);
   }
   else {
      if ( !( repository = malloc( sizeof( char) * strlen(* argv) + 15))) {
         fprintf( stderr, "\nmain.c: malloc error");
         safe_bail_out( -3);
      }
      strcpy( repository, *argv);
      strcat( repository, "/html_analyzer");
   }

   /* 
    * create the tempory directory; this enables all future completeness
    * processing as well as easy clean up when done
    */ 
   clean_dir( repository);
   if ( mkdir( repository, S_IWRITE | S_IREAD | S_IEXEC)) {
      fprintf( stderr, "\nmain.c: unable to make %s directory", repository);
      safe_bail_out( -3);
   }
   
   /*
    * call extract-links via recurse_fcn to extract the links from
    * all "html" files in the directory hierarchy; 
    * call html_analyzer to perfom specified tests
    */
   html_analyzer( directory, supress);

   safe_bail_out(0);
}
/*
 * these definitions are necessary to compile with the new www library
 */
#include "../libwww-2.09a/HTUtils.h"
#include "../libwww-2.09a/HTString.h"
#include "../libwww-2.09a/tcp.h"
#include "../libwww-2.09a/HTTCP.h"
#include "../libwww-2.09a/HTUtils.h"
#include "../libwww-2.09a/HTAnchor.h"
#include "../libwww-2.09a/HTParse.h"
#include "../libwww-2.09a/HTAccess.h"
#include "../libwww-2.09a/HTML.h"
#include "../libwww-2.09a/HText.h"
#include "../libwww-2.09a/HTList.h"

HText           *HTMainText     = 0;
HTParentAnchor  *HTMainAnchor   = 0;
char            *HTAppName      = 0;
char            *HTAppVersion   = 0;
int              bypass_format_handling = 0;
int              force_dump_to_file     = 0;
char            *force_dump_filename    = 0;

char    *HText_getText (HText *me) {}
HText   *HText_new (HTParentAnchor *anchor) {}
HText   *HText_new2 (HTParentAnchor *anchor, HTStream *output_stream) {}
void    gui_notify_progress() {}
/* was void loadin-... */
/*int    loading_inlined_images() {}*/
void	mo_gui_done_with_icon() {}
void 	mo_gui_notify_progress (char *text) {}
int 	mo_gui_check_icon () {}
void 	mo_gui_clear_icon () {}
void    HText_appendCharacter (HText *text, char ch) {}
void    HText_appendParagraph (HText *text) {}
void    HText_appendText (HText *text, char *str) {}
void    HText_beginAnchor (HText *text, char *anc) {}
void    HText_beginAppend (HText *text) {}
void    HText_endAnchor (HText * text) {}
void    HText_endAppend (HText *text) {}
void    HText_dump (HText *me) {}
void    HText_free (HText *self) {}
BOOL    HText_select (HText *text) {}
BOOL    HText_selectAnchor (HText *text, HTChildAnchor *anchor) {}
static void     new_chunk (HText *text) {}
HTParentAnchor *HText_nodeAnchor (HText *me) {}
